# ALMA Data Reduction Script

### This is a QA2-like script for calibration for the Band-to-Band
### testing dataset for VY CMa observed at band 9 during the 2017 early
### Cycle 5 SB testing for the High-Frequency Long Baselines EOC campaign,
### Here, the B2B uses phase transfer from a phase calibrator observed at band 4. 
### This is NOT a Science Verification dataset - these are test data

### Notes will be also added during the steps to help/inform inquisitive
### users. Note also, these data were taken with SBs but specifically for
### testing purposes. These data were not, and are not science verification
### datasets and thus do not fully follow a 'standard' observational sequence.

### For further information please contact:
### yasaki@alma.cl  JAO/NAOJ
### lmaud@eso.org   ESO 
###
### If using these data please reference:
### Asaki et al, 2020 ApJS, 247, 23A
### and the B2B papers therin

### Original B2B Script Author Luke Maud - ESO
### Hardcoded script Final Version
### Yoshiharu Asaki JAO, Anita Richards, Luke Maud ESO
### 13/03/2020 - CASA 5.6.1-8 version



thesteps = []
step_title = {0: 'Import of the ASDMs',
              1: 'Fix of SYSCAL table times',
              2: 'Listobs',
              3: 'A priori flagging',
              4: 'Generation and time averaging of the WVR cal table',
              5: 'Generation of the Tsys cal table',
              6: 'Generation of the antenna position cal table',
              7: 'Application of the WVR, Tsys and antpos cal tables',
              8: 'Split out science SPWs and time average',
              9: 'Listobs, and save original flags',
              10: 'Initial flagging',
              11: 'Putting a model for the flux calibrator(s)',
              12: 'Save flags before bandpass cal',
              13: 'Bandpass Calibration',
              14: 'Save flags before Gain calibration',
              15: 'Gain Calibration - DGC - LF sols to cross apply to HF',
              16: 'Gain Calibration - Phase and Amp cal',
              17: 'Save flags before applycal',
              18: 'Application of the bandpass and gain cal tables',
              19: 'Split out target with mstransform with/wout chan averaging'
              }


if 'applyonly' not in globals(): applyonly = True
try:
  print 'List of steps to be executed ...', mysteps
  thesteps = mysteps
except:
  print 'global variable mysteps not set.'
if (thesteps==[]):
  thesteps = range(0,len(step_title))
  print 'Executing all steps: ', thesteps

# The Python variable 'mysteps' will control which steps
# are executed when you start the script using
#   execfile('scriptForCalibration.py')
# e.g. setting
#   mysteps = [2,3,4]
# before starting the script will make the script execute
# only steps 2, 3, and 4
# Setting mysteps = [] will make it execute all steps.

import re

import os

import casadef

if applyonly != True: es = aU.stuffForScienceDataReduction() 


if re.search('^5.6.1', '.'.join([str(i) for i in cu.version().tolist()[:-1]])) == None:
 sys.exit('ERROR: PLEASE USE THE SAME VERSION OF CASA THAT YOU USED FOR GENERATING THE SCRIPT: 5.6.1')


 
# CALIBRATE_AMPLI: 
# CALIBRATE_ATMOSPHERE: 
# CALIBRATE_BANDPASS: J0522-3627
# CALIBRATE_DIFFGAIN: J0522-3627
# CALIBRATE_FLUX: J0522-3627
# CALIBRATE_FOCUS: 
# CALIBRATE_PHASE: J0725-2640
# CALIBRATE_POINTING: J0747-3310
# CALIBRATE_POLARIZATION: 
# OBSERVE_CHECK: J0816-2421
# OBSERVE_TARGET: VY CMa


# Using reference antenna = DV07

#####################################
#
# Import of the ASDM
#
#####################################

mystep = 0
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]


  ## this step will always remove the MS - as re-running the complex conjugate correction
  ## on a pre-exiting MS will incorrect change the phases again
  os.system('rm -rf uid___A002_Xc660ef_X2d26.ms')
  os.system('mv uid___A002_Xc660ef_X2d26.asdm.sdm uid___A002_Xc660ef_X2d26')
  if os.path.exists('uid___A002_Xc660ef_X2d26.ms') == False:
    importasdm('uid___A002_Xc660ef_X2d26', asis='*', bdfflags=True, lazy=False, process_caldevice=False)
    if not os.path.exists('uid___A002_Xc660ef_X2d26.ms.flagversions'):
      print 'ERROR in importasdm. Output MS is probably not useful. Will stop here.'
      thesteps = []

    
  if applyonly != True: es.fixForCSV2555('uid___A002_Xc660ef_X2d26.ms')



#####################################################
# Low frequency SPWs: to make complex conjugate for Low Freq SPWs
# This part has been prepared for the trouble shooting of ICT-18888
#####################################################

  tb.open('uid___A002_Xc660ef_X2d26.ms', nomodify=False)
  for desc_id in range(48, 75): # i.e. the low SPW only which are 'flipped'
    raw_data = tb.query('DATA_DESC_ID==%d' % desc_id)
    uncal_data = raw_data.getcol('DATA')
    raw_data.putcol('DATA', uncal_data.conj())
    raw_data.close()
  tb.close()





  
#####################################
#
# Fix of SYSCAL table times
#
#####################################
mystep = 1
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  from recipes.almahelpers import fixsyscaltimes
  fixsyscaltimes(vis = 'uid___A002_Xc660ef_X2d26.ms')




print "# A priori calibration"


#####################################
#
# Listobs
#
#####################################
mystep = 2
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf '+'uid___A002_Xc660ef_X2d26.ms.listobs')
  listobs(vis = 'uid___A002_Xc660ef_X2d26.ms',
    listfile = 'uid___A002_Xc660ef_X2d26.ms.listobs')
  
  plotants(vis='uid___A002_Xc660ef_X2d26.ms', 
    figfile='uid___A002_Xc660ef_X2d26.ms.antpos.png')


  # Extra note - there will be 8 LF spectral windows, this is a 'feature' of the testing B2B
  # mode when walsh switching is employed (to give 2x BW at the HF). Here only 4 of the 
  # LF SPWs are actaully containing real data

  
#####################################
#
# A priori flagging
#
#####################################

mystep = 3
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  flagdata(vis = 'uid___A002_Xc660ef_X2d26.ms', 
      mode = 'manual', autocorr = True, flagbackup = False)

  flagdata(vis = 'uid___A002_Xc660ef_X2d26.ms',
      mode = 'manual',  intent = '*POINTING*,*ATMOSPHERE*', flagbackup = False)


  flagcmd(vis = 'uid___A002_Xc660ef_X2d26.ms',
    inpmode = 'table',
    useapplied = True,
    action = 'plot',
    plotfile = 'uid___A002_Xc660ef_X2d26.ms.flagcmd.png')
  
  flagcmd(vis = 'uid___A002_Xc660ef_X2d26.ms',
    inpmode = 'table',
    useapplied = True,
    action = 'apply')


  ## For the VY CMa observation, the first CALIBRATE_DIFFGAIN scan are problematic.
  flagdata(vis='uid___A002_Xc660ef_X2d26.ms', 
           mode = 'manual',
           scan='6,7',
           flagbackup = False)
  

#####################################
#
# Generation and time averaging of the WVR cal table
#
#####################################
mystep = 4
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]


  os.system('rm -rf '+'uid___A002_Xc660ef_X2d26.ms.wvr')   
  os.system('rm -rf '+'uid___A002_Xc660ef_X2d26.ms.wvrgcal') 
  

  mylogfile = casalog.logfile()
  casalog.setlogfile('uid___A002_Xc660ef_X2d26.ms.wvrgcal')
  wvrgcal(vis = 'uid___A002_Xc660ef_X2d26.ms',
          caltable = 'uid___A002_Xc660ef_X2d26.ms.wvr')
 
  # note already flagged some antennas so they have warning and no WVR solns

  casalog.setlogfile(mylogfile)


  # Extra Notes
  # This makes the WVR solutions plots for both frequencies,
  # those for the higher frequency SHOULD look to have a larger phase variations (band 4 to band 9, Hf/Lf is more than a factor of 4)
  # as phase variations scales linearly with frequency

  
  if applyonly != True: aU.plotWVRSolutions(caltable='uid___A002_Xc660ef_X2d26.ms.wvr', spw='29',
                                            antenna='DV07', yrange=[-199,199],subplot=22,
                                            interactive=False, figfile='uid___A002_Xc660ef_X2d26.ms.wvr.low.plots/'+'uid___A002_Xc660ef_X2d26.ms.wvr') 

  if applyonly != True: aU.plotWVRSolutions(caltable='uid___A002_Xc660ef_X2d26.ms.wvr', spw='59',
                                            antenna='DV07', yrange=[-199,199],subplot=22,
                                            interactive=False, figfile='uid___A002_Xc660ef_X2d26.ms.wvr.high.plots/'+'uid___A002_Xc660ef_X2d26.ms.wvr') 

#####################################
#
# Generation of the Tsys cal table
#
#####################################
mystep = 5
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf '+'uid___A002_Xc660ef_X2d26.ms.tsys') 
  gencal(vis = 'uid___A002_Xc660ef_X2d26.ms',
	 caltable = 'uid___A002_Xc660ef_X2d26.ms.tsys',
	 caltype = 'tsys')


 
  # Flagging edge channels of Tsys

  # Extra notes - 8 HF SPWs, 4 'real' LF SPWs + 4 Redundant LF SPW
  # due to how the data and Tsys are saved - some get data, some get Tsys
  # but are the 'same' SPW but 'mirrored' due to the Walsh Switching effecting
  # the LF SPWs


  flagdata(vis = 'uid___A002_Xc660ef_X2d26.ms.tsys',
	   mode = 'manual',
	   spw = '7:0~7;120~127,17:0~7;120~127,19:0~7;120~127,21:0~7;120~127,23:0~7;120~127,25:0~7;120~127,27:0~7;120~127,29:0~7;120~127,31:0~7;120~127,48:0~7;120~127,50:0~7;120~127,53:0~7;120~127,55:0~7;120~127,57:0~7;120~127,59:0~7;120~127,61:0~7;120~127', 
	   flagbackup = False)

  ## Unavoidable to open the Tsys tables and copy the matching 'pair' 
  ## Tsys to the 'empty' Tsys SPWs, as these contain the 'real' data
  ## and allows later SPW Tsys mapping to work correctly

  tb.open('uid___A002_Xc660ef_X2d26.ms.tsys', nomodify=False)
  nrows = tb.nrows()
  for spw_id in [7, 48]:
      print 'Tsys information modification: Spw %d' % spw_id
      raw_data_1 = tb.query('SPECTRAL_WINDOW_ID==%d' % spw_id)
      if spw_id == 7:
        mod_spw_id = 53
      else:
        mod_spw_id = 50
      raw_data_2 = tb.query('SPECTRAL_WINDOW_ID==%d' % mod_spw_id)

      tsys_data_1 = raw_data_1.getcol('FPARAM')
      tsys_data_2 = raw_data_2.getcol('FPARAM')
      ant1_data_1 = raw_data_1.getcol('ANTENNA1')
      ant1_data_2 = raw_data_2.getcol('ANTENNA1')
      time_data_1 = raw_data_1.getcol('TIME')
      time_data_2 = raw_data_2.getcol('TIME')

      for irow in range(tsys_data_1.shape[2]):
        for jrow in range(irow,tsys_data_2.shape[2]):
          if ant1_data_1[irow] == ant1_data_2[jrow] and time_data_1[irow] == time_data_2[jrow]:
            for ich in range(128):
              tsys_data_2[0][ich][jrow] = tsys_data_1[0][ich][irow]
              tsys_data_2[1][ich][jrow] = tsys_data_1[1][ich][irow]
            break
          else:
            print '#### WARNING (%d  %d) ####' % (irow, jrow)
      raw_data_2.putcol('FPARAM', tsys_data_2)
      raw_data_2.close()
  tb.close()

  if applyonly != True: es.checkCalTable('uid___A002_Xc660ef_X2d26.ms.tsys', msName='uid___A002_Xc660ef_X2d26.ms', interactive=False) 


#####################################
#
# Generation of the antenna position cal table
#
#####################################

mystep = 6
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]
  

  
  os.system('rm -rf uid___A002_Xc660ef_X2d26.ms.antpos') 
  gencal(vis = 'uid___A002_Xc660ef_X2d26.ms',
	 caltable = 'uid___A002_Xc660ef_X2d26.ms.antpos',
	 caltype = 'antpos',
	 antenna = 'DA41,DA43,DA44,DA45,DA46,DA47,DA48,DA49,DA50,DA51,DA52,DA53,DA54,DA55,DA56,DA57,DA58,DA59,DA60,DA63,DA64,DA65,DV01,DV04,DV07,DV08,DV10,DV11,DV12,DV13,DV14,DV15,DV17,DV19,DV22,DV23,DV24,DV25,PM01,PM02,PM03,PM04',
	 #parameter = [-1.78142e-05,8.30644e-04,4.10085e-04,-4.00916e-05,6.65741e-04,9.64524e-05,-5.13782e-04,8.87891e-04,-1.47149e-07,1.53058e-04,8.01244e-05,2.52155e-04,2.26063e-04,-5.81427e-04,-2.66012e-05,-2.25006e-05,5.91375e-04,6.45005e-04,6.83242e-04,-1.52784e-03,-4.22246e-04,3.30791e-05,3.94730e-04,4.06143e-04,2.11079e-04,2.67035e-04,5.99129e-04,-2.24102e-04,4.34216e-04,4.66974e-04,6.50448e-04,-9.34455e-04,-3.59355e-04,3.62997e-04,-1.46167e-03,-8.35765e-04,7.15067e-04,-3.41355e-03,-1.45851e-03,-1.83205e-04,6.40403e-04,2.95802e-04,-9.90769e-05,3.98024e-04,1.51837e-04,-1.72916e-04,7.66136e-04,5.05006e-04,1.34758e-04,9.41884e-05,5.59557e-04,3.13418e-05,-1.13010e-03,-9.20565e-04,4.06404e-04,3.42857e-04,-2.14315e-04,-3.06051e-04,-2.69569e-03,-6.40729e-04,-3.16965e-04,-7.83368e-05,2.37111e-04,1.41396e-04,-6.61986e-04,1.48773e-04,1.09947e-04,-2.76429e-04,-3.72016e-04,-2.38231e-04,-3.02599e-03,-3.10416e-04,1.29661e-04,-1.47888e-04,1.60294e-04,-5.98185e-04,-5.09675e-03,-2.33882e-03,4.00050e-04,4.65104e-04,5.80347e-04,-4.10370e-04,9.58987e-04,1.10625e-03,2.90854e-04,-8.97086e-04,-4.04840e-04,1.36811e-04,9.79351e-05,-6.51285e-04,-6.75720e-04,1.60965e-03,7.99060e-04,-1.01829e-03,1.96660e-03,4.95779e-04,1.31668e-05,5.12543e-04,4.97609e-04,-8.77078e-05,4.78648e-04,2.42355e-04,-1.84960e-03,1.08218e-03,2.98598e-04,-2.78929e-04,1.08367e-03,9.43055e-04,-2.06528e-04,1.42435e-03,1.16265e-03,5.01624e-05,2.10368e-04,7.30627e-05,-6.40727e-05,4.88757e-04,4.37836e-04,-9.17429e-05,3.21982e-04,2.89154e-04,-1.20766e-04,1.02105e-03,1.24334e-04,-6.39522e-04,7.31457e-04,-3.68585e-04]) # values 
         parameter = [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
  # 0's no correction/update --> negligable difference - alsod for HLTau data it was better without - so ASDM was updated ok
  
  
  # antenna x_offset y_offset z_offset total_offset baseline_date
  #DV08    -5.98185e-04   -5.09675e-03   -2.33882e-03    5.63957e-03      2017-10-06 03:03:50
  # DA54     7.15067e-04   -3.41355e-03   -1.45851e-03    3.78033e-03      2017-10-06 03:03:50
  # DV04    -2.38231e-04   -3.02599e-03   -3.10416e-04    3.05119e-03      2017-10-06 03:03:50
  # DA63    -3.06051e-04   -2.69569e-03   -6.40729e-04    2.78765e-03      2017-10-06 03:03:50
  # DV15    -1.01829e-03    1.96660e-03    4.95779e-04    2.26941e-03      2017-11-26 00:12:34
  # DV22    -1.84960e-03    1.08218e-03    2.98598e-04    2.16363e-03      2017-11-26 00:12:34
  # DV14    -6.75720e-04    1.60965e-03    7.99060e-04    1.91991e-03      2017-11-16 08:09:25
  # DV24    -2.06528e-04    1.42435e-03    1.16265e-03    1.85019e-03      2017-10-17 01:49:03
  # DA48     6.83242e-04   -1.52784e-03   -4.22246e-04    1.72610e-03      2017-10-06 03:03:50
  # DA53     3.62997e-04   -1.46167e-03   -8.35765e-04    1.72242e-03      2017-10-06 03:03:50
  # DV11    -4.10370e-04    9.58987e-04    1.10625e-03    1.52048e-03      2017-11-26 00:12:34
  # DV23    -2.78929e-04    1.08367e-03    9.43055e-04    1.46338e-03      2017-11-16 08:09:25
  # DA59     3.13418e-05   -1.13010e-03   -9.20565e-04    1.45792e-03      2017-10-06 03:03:50
  # DA52     6.50448e-04   -9.34455e-04   -3.59355e-04    1.19391e-03      2017-10-06 03:03:50
  # PM04    -6.39522e-04    7.31457e-04   -3.68585e-04    1.03917e-03      2017-11-26 00:12:34
  # PM03    -1.20766e-04    1.02105e-03    1.24334e-04    1.03566e-03      2017-11-16 08:09:25
  # DV12     2.90854e-04   -8.97086e-04   -4.04840e-04    1.02628e-03      2017-10-06 03:03:50
  # DA44    -5.13782e-04    8.87891e-04   -1.47149e-07    1.02583e-03      2017-11-26 00:12:34
  # DA57    -1.72916e-04    7.66136e-04    5.05006e-04    9.33754e-04      2017-11-16 08:09:25
  # DA41    -1.78142e-05    8.30644e-04    4.10085e-04    9.26529e-04      2017-11-26 00:12:34
  # DA47    -2.25006e-05    5.91375e-04    6.45005e-04    8.75364e-04      2017-11-16 08:09:25
  # DV10     4.00050e-04    4.65104e-04    5.80347e-04    8.44491e-04      2017-11-26 00:12:34
  # DA55    -1.83205e-04    6.40403e-04    2.95802e-04    7.28820e-04      2017-11-16 08:09:25
  # DV17     1.31668e-05    5.12543e-04    4.97609e-04    7.14484e-04      2017-11-16 08:09:25
  # DA65     1.41396e-04   -6.61986e-04    1.48773e-04    6.93074e-04      2017-11-16 08:09:25
  # DA50     2.11079e-04    2.67035e-04    5.99129e-04    6.89070e-04      2017-11-26 00:12:34
  # DA51    -2.24102e-04    4.34216e-04    4.66974e-04    6.75892e-04      2017-11-16 08:09:25
  # DA43    -4.00916e-05    6.65741e-04    9.64524e-05    6.73885e-04      2017-11-26 00:12:34
  # DV13     1.36811e-04    9.79351e-05   -6.51285e-04    6.72667e-04      2017-11-26 00:12:34
  # PM01    -6.40727e-05    4.88757e-04    4.37836e-04    6.59309e-04      2017-11-16 08:09:25
  # DA46     2.26063e-04   -5.81427e-04   -2.66012e-05    6.24396e-04      2017-11-16 08:09:25
  # DA58     1.34758e-04    9.41884e-05    5.59557e-04    5.83211e-04      2017-11-26 00:12:34
  # DA60     4.06404e-04    3.42857e-04   -2.14315e-04    5.73277e-04      2017-11-16 08:09:25
  # DA49     3.30791e-05    3.94730e-04    4.06143e-04    5.67325e-04      2017-11-16 08:09:25
  # DV19    -8.77078e-05    4.78648e-04    2.42355e-04    5.43629e-04      2017-11-16 08:09:25
  # DV01     1.09947e-04   -2.76429e-04   -3.72016e-04    4.76337e-04      2017-10-06 03:03:50
  # PM02    -9.17429e-05    3.21982e-04    2.89154e-04    4.42379e-04      2017-11-16 08:09:25
  # DA56    -9.90769e-05    3.98024e-04    1.51837e-04    4.37372e-04      2017-11-16 08:09:25
  # DA64    -3.16965e-04   -7.83368e-05    2.37111e-04    4.03516e-04      2017-11-16 08:09:25
  # DA45     1.53058e-04    8.01244e-05    2.52155e-04    3.05662e-04      2017-11-16 08:09:25
  # DV07     1.29661e-04   -1.47888e-04    1.60294e-04    2.53726e-04      2017-11-16 08:09:25
  # DV25     5.01624e-05    2.10368e-04    7.30627e-05    2.28274e-04      2017-11-16 08:09:25\n"
  


            
#####################################
#
# Application of the WVR, Tsys and antpos cal tables
#
#####################################
mystep = 7
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  from recipes.almahelpers import tsysspwmap
  tsysmap = tsysspwmap(vis = 'uid___A002_Xc660ef_X2d26.ms', tsystable = 'uid___A002_Xc660ef_X2d26.ms.tsys', tsysChanTol = 1)
  

  # J0522-3627 has a tsys
  applycal(vis = 'uid___A002_Xc660ef_X2d26.ms',
    field = '0',
    spw = '29,31,33,35,37,39,41,43,59,61,63,65,67,69,71,73',
    gaintable = ['uid___A002_Xc660ef_X2d26.ms.tsys', 'uid___A002_Xc660ef_X2d26.ms.wvr', 'uid___A002_Xc660ef_X2d26.ms.antpos'],
    gainfield = ['0', '', ''],
    interp = 'linear,linear',
    spwmap = [tsysmap,[],[]],
    calwt = True,
    flagbackup = False)

  # J0725-2640 has a tsys
  applycal(vis = 'uid___A002_Xc660ef_X2d26.ms',
    field = '2',
    spw = '29,31,33,35,37,39,41,43,59,61,63,65,67,69,71,73',
    gaintable = ['uid___A002_Xc660ef_X2d26.ms.tsys', 'uid___A002_Xc660ef_X2d26.ms.wvr', 'uid___A002_Xc660ef_X2d26.ms.antpos'],
    gainfield = ['2', '', ''],
    interp = 'linear,linear',
    spwmap = [tsysmap,[],[]],
    calwt = True,
    flagbackup = False)

  # J0816-2421 does not has a tsys - use one from the target VY CMa
  applycal(vis = 'uid___A002_Xc660ef_X2d26.ms',
    field = '3',
    spw = '29,31,33,35,37,39,41,43,59,61,63,65,67,69,71,73',
    gaintable = ['uid___A002_Xc660ef_X2d26.ms.tsys', 'uid___A002_Xc660ef_X2d26.ms.wvr', 'uid___A002_Xc660ef_X2d26.ms.antpos'],
    gainfield = ['4', '', ''],
    interp = 'linear,linear',
    spwmap = [tsysmap,[],[]],
    calwt = True,
    flagbackup = False)


  # VY CMa has a tsys
  applycal(vis = 'uid___A002_Xc660ef_X2d26.ms',
    field = '4',
    spw = '29,31,33,35,37,39,41,43,59,61,63,65,67,69,71,73',
    gaintable = ['uid___A002_Xc660ef_X2d26.ms.tsys', 'uid___A002_Xc660ef_X2d26.ms.wvr', 'uid___A002_Xc660ef_X2d26.ms.antpos'],
    gainfield = ['4', '', ''],
    interp = 'linear,linear',
    spwmap = [tsysmap,[],[]],
    calwt = True,
    flagbackup = False)
  

  if applyonly != True: es.getCalWeightStats('uid___A002_Xc660ef_X2d26.ms')   


#####################################
#
# Split out science SPWs and time average
#
#####################################

mystep = 8
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]


  # Extra note:
  # This is the full split file - with corrected SPW orders this can
  # keep the high and low freq together, high 0,1,2,3,4,5,6,7 and low 8,9,10,11
  # The 'real' LF SPWs are 59,65,69,71 with data
 
  os.system('rm -rf uid___A002_Xc660ef_X2d26.ms.split') 
  os.system('rm -rf uid___A002_Xc660ef_X2d26.ms.split.flagversions') 

  listOfIntents = ['CALIBRATE_BANDPASS#ON_SOURCE',
                   'CALIBRATE_FLUX#ON_SOURCE',
                   'CALIBRATE_PHASE#ON_SOURCE',
                   'CALIBRATE_DIFFGAIN#SIGNAL',
                   'CALIBRATE_DIFFGAIN#REFERENCE',
                   'OBSERVE_CHECK_SOURCE#ON_SOURCE',
                   'OBSERVE_TARGET#ON_SOURCE']

  split(vis = 'uid___A002_Xc660ef_X2d26.ms',
    outputvis = 'uid___A002_Xc660ef_X2d26.ms.split',
    datacolumn = 'corrected',
    spw = '29,31,33,35,37,39,41,43,59,65,69,71',    # First 8 spws for HF while the last 4 for LF
    intent = ','.join(listOfIntents),
    keepflags = True)


 
  ## fix vis phase tracing center - slight offset in VY CMa for strong point to centre (later selfcal requires this)
  fixvis(vis='uid___A002_Xc660ef_X2d26.ms.split', field='4', reuse=True,
         phasecenter='J2000 07h22m58.32600s -25d46m03.03550s', datacolumn='all')


#####################################
#
# Listobs, and save original flags
#
#####################################

mystep = 9
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf '+'uid___A002_Xc660ef_X2d26.ms.split.listobs')
  listobs(vis = 'uid___A002_Xc660ef_X2d26.ms.split',
    listfile = 'uid___A002_Xc660ef_X2d26.ms.split.listobs')
  
  
  if not os.path.exists('uid___A002_Xc660ef_X2d26.ms.split.flagversions/Original.flags'):
    flagmanager(vis = 'uid___A002_Xc660ef_X2d26.ms.split',
      mode = 'save',
      versionname = 'Original')
  

#####################################
#
# Initial flagging
#
#####################################

mystep = 10
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]


  # Flagging shadowed data
  
  flagdata(vis = 'uid___A002_Xc660ef_X2d26.ms.split',
    mode = 'shadow',
    flagbackup = False)
  

  # Flagging edge channels

  
  flagdata(vis = 'uid___A002_Xc660ef_X2d26.ms.split',
      mode = 'manual',
      spw = '0:0~7;120~127,1:0~7;120~127,8:0~7;120~127,9:0~63;1800~1919,10:0~63;1800~1919,11:0~63;1800~1919',# take out all edges of LF 
      flagbackup = False)


  
  # Birdie in Spw 2 at 658.8 GHz in scan 22 (not superimposed on the H2O masers)

  flagdata(vis = 'uid___A002_Xc660ef_X2d26.ms.split',
      mode = 'manual',
      scan = '22',
      spw = '2:445~448',
      flagbackup = False)

  flagdata(vis = 'uid___A002_Xc660ef_X2d26.ms.split',
      mode = 'manual',
      scan = '22',
      spw = '2:1918~1919',
      flagbackup = False)

  # Flagging Band 4 Spw9 sharp drop
  # This is caused by mismatch of the frequency resolution 
  # between science FDM Spw (1920 ch) and Tsys Spw TDM (128 ch). (e.g. PRTSPR-36441)

  flagdata(vis = 'uid___A002_Xc660ef_X2d26.ms.split',
      mode = 'manual',
      spw = '9:270~389',
      flagbackup = False)



#####################################
#
# Putting a model for the flux calibrator(s)
#
#####################################

mystep = 11
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

 

  # Use spectral index
  setjy(vis = 'uid___A002_Xc660ef_X2d26.ms.split',
            standard = 'manual',
            field = '0',  # in high band only we need the flux cal 
            fluxdensity = [3.40621475783, 0, 0, 0],
            spix = -0.257996321858,
            reffreq='664.9114038GHz',
            spw = '0,1,2,3,4,5,6,7')


#####################################
#
# Save flags before bandpass cal
#
#####################################

mystep = 12
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]
  
  flagmanager(vis = 'uid___A002_Xc660ef_X2d26.ms.split',
    mode = 'save',
    versionname = 'BeforeBandpassCalibration')
  

#####################################
#
# Bandpass calibration   
#
#####################################

mystep = 13
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]
 
  ## Extra note:
  ## The DGC source is also used as the bandpass. It was observed at the HF as a dedicated 
  ## bandpass source, but at the low frequency the DGC scans must be used as there is no
  ## dedicated LF bandpass target
  

  os.system('rm -rf uid___A002_Xc660ef_X2d26.ms.split.ap_pre_bandpass') 
  gaincal(vis = 'uid___A002_Xc660ef_X2d26.ms.split',
	  caltable = 'uid___A002_Xc660ef_X2d26.ms.split.ap_pre_bandpass',
	  field = '0',
	  spw = '0,1,2,3,4,5,6,7,8,9,10,11', ## all spw for HF and LF
	  solint = 'int',
	  minsnr=3.0,
	  refant = 'DV07',
	  calmode = 'p',
	  append=True)
  
  if applyonly != True: es.checkCalTable('uid___A002_Xc660ef_X2d26.ms.split.ap_pre_bandpass', msName='uid___A002_Xc660ef_X2d26.ms.split', interactive=False)

  # now the actual BP solution
  os.system('rm -rf uid___A002_Xc660ef_X2d26.ms.split.bandpass') 
  bandpass(vis = 'uid___A002_Xc660ef_X2d26.ms.split',
	   caltable = 'uid___A002_Xc660ef_X2d26.ms.split.bandpass',
	   field = '0',
	   solint = 'inf,15.625MHz',  # Average FDM to TDM resolution
	   combine = 'scan', 
	   refant = 'DV07',
	   solnorm = True,
	   bandtype = 'B',
           minsnr= 3.0, 
	   gaintable = 'uid___A002_Xc660ef_X2d26.ms.split.ap_pre_bandpass')
		    
  if applyonly != True: es.checkCalTable('uid___A002_Xc660ef_X2d26.ms.split.bandpass', msName='uid___A002_Xc660ef_X2d26.ms.split', interactive=False)


#####################################
#
# Save flags before gain cal
#
#####################################

mystep = 14
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]
  
  flagmanager(vis = 'uid___A002_Xc660ef_X2d26.ms.split',
    mode = 'save',
    versionname = 'BeforeGainCalibration')


#####################################
#
# Gain calibration DGC - LF to HF cross apply
#
#####################################


  # This is the DGC step - this is the unique step for the B2B modes
  # This involves solving for the LF to HF instrumental offset using the DiffGainCal (DGC)
  # source. Ultimately there are two ways to do this:

  # A LF to HF cross apply followed by the  offset solution (cross apply method)
  # This EXPLICITLY solves the DGC offset and is a solution to be applied
  # to the HF data in the final applycal stages  - THIS IS WHAT IS USED FOR THESE DATA 
  # AS THE MOST ROBUST B2B CALIBRATION TECHNIQUE FOR LB

  #### more infomation on the phaseDiff method (NOT USED) ####

  # The phaseDiff method - as each DGC 'sequence' is an observation for a short total time (<few mins)
  # but cycled fast (<30-40s between the HF and LF) to produce interleaved observations 
  # of the same target but alternate frequencies, then _IF_ the cycling is
  # fast enough to assume 'pairs' of the HF and LF sub-scans are really looking through the
  # 'same' atmopshere, then one can employ a simple phaseDiff solution at 
  # both the HF and the LF, i.e. single solutions for the HF and LF independently
  # aligning all SPW. For B2B in all calibration or application the HF or the LF
  # phaseDiff solutions must be applied as the assumption is that the B2B offset
  # is not dierectly solved, but _IMPLICIT_ from all SPW being alligned at a time 'T'
  # which is the mid-time point of the DGC used for the phaseDiff solutions

  # In practice, the phaseDiff and cross apply are equivalent 
  # however the phaseDiff can be senstive to poorer atmospheric conditions, whereby
  # many DGC sequences are used, due to a separation in time of more than 10 
  # minutes, such that it _IS_ possible for the HF to phase wrap, while the corresponding
  # LF does not wrap. This meeans the phaseDiff solutions do not produce 
  # and implicit correction and there is a resulting offset after the application.
  # This can be solved then with another DGC HF offset solve, which again is an 
  # _EXPLICIT_ correction for the offset. This is avoidable if using only ONE DGC sequence
  # lasting ~5-6minutes, where the phase should not wrap - if it does then likely the phase
  # stability is so bad, that you shouldn't be observing !



  # Applying a phasediff to the LF as this gives us better SNR



mystep = 15
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  # do a phase offset for LF so all SPW can be merged for better SNR 

  os.system('rm -rf uid___A002_Xc660ef_X2d26.ms.split.DGCphaselow_phasediff') 
  gaincal(vis = 'uid___A002_Xc660ef_X2d26.ms.split',
    caltable = 'uid___A002_Xc660ef_X2d26.ms.split.DGCphaselow_phasediff',
    field =  '0',   
    scan='6,8,10,12,14,16,38,40,42,44,46,48',
    solint = 'inf',  # 
    refant = 'DV07',
    gaintype = 'G',
    calmode = 'p',
    combine='scan',   
    minsnr=3.0,
    gaintable = 'uid___A002_Xc660ef_X2d26.ms.split.bandpass')
    
  if applyonly != True: es.checkCalTable('uid___A002_Xc660ef_X2d26.ms.split.DGCphaselow_phasediff', msName='uid___A002_Xc660ef_X2d26.ms.split', interactive=False) 


  ## Solve the  LF phases in time (combine with SPW)
  os.system('rm -rf uid___A002_Xc660ef_X2d26.ms.split.DGCphaselow_inf') 

  gaincal(vis = 'uid___A002_Xc660ef_X2d26.ms.split',
    caltable = 'uid___A002_Xc660ef_X2d26.ms.split.DGCphaselow_inf',
    field = '0',    
    scan='6,8,10,12,14,16,38,40,42,44,46,48', 
    solint = 'inf',  # inf as we interpolate the solution
    refant = 'DV07',
    gaintype = 'G',
    calmode = 'p',
    minsnr=3.0,
    combine='spw', 
    interp=['linear','linear'],
    gaintable = ['uid___A002_Xc660ef_X2d26.ms.split.bandpass','uid___A002_Xc660ef_X2d26.ms.split.DGCphaselow_phasediff'],
    gainfield = ['0','0'])

  if applyonly != True: es.checkCalTable('uid___A002_Xc660ef_X2d26.ms.split.DGCphaselow_inf', msName='uid___A002_Xc660ef_X2d26.ms.split', interactive=False) 


  ## apply LF to HF cross apply and solve the offset - for each HF SPW _alone_
  ## and using INF (but no scan crossing) this is a testing step to
  ## see how well the temporal phases are solved - these should be 'flat' with time after application
  ## if LF to HF transfer was reasonable

  os.system('rm -rf uid___A002_Xc660ef_X2d26.ms.split.DGCoffset_inf') 
  gaincal(vis = 'uid___A002_Xc660ef_X2d26.ms.split',
    caltable = 'uid___A002_Xc660ef_X2d26.ms.split.DGCoffset_inf',
    field = '0',  
    scan='7,9,11,13,15,39,41,43,45,47',
    spw='0,1,2,3,4,5,6,7',
    solint = 'inf',  #
    refant = 'DV07',
    gaintype = 'G',
    calmode = 'p',
    minsnr=3.0,
    interp=['linear','linearPD'], 
    spwmap=[[],[8,8,8,8,8,8,8,8]], 
    gainfield = ['0','0'],  
    gaintable = ['uid___A002_Xc660ef_X2d26.ms.split.bandpass','uid___A002_Xc660ef_X2d26.ms.split.DGCphaselow_inf'])

  if applyonly != True: es.checkCalTable('uid___A002_Xc660ef_X2d26.ms.split.DGCoffset_inf', msName='uid___A002_Xc660ef_X2d26.ms.split', interactive=False)



  # apply the LF to the HF and find the offset SINGLE 
  os.system('rm -rf uid___A002_Xc660ef_X2d26.ms.split.DGCoffset_single') 
  gaincal(vis = 'uid___A002_Xc660ef_X2d26.ms.split',
    caltable = 'uid___A002_Xc660ef_X2d26.ms.split.DGCoffset_single',
    field = '0',  #
    scan='7,9,11,13,15,39,41,43,45,47',
    spw='0,1,2,3,4,5,6,7',
    solint = 'inf',  #
    refant = 'DV07',
    gaintype = 'G',
    calmode = 'p',
    minsnr=3.0,
    combine='scan',# inf and scan combine for a single offset - one single HF solution for offset
    interp=['linear','linearPD'], 
    spwmap=[[],[8,8,8,8,8,8,8,8]], 
    gainfield = ['0','0'],  
    gaintable = ['uid___A002_Xc660ef_X2d26.ms.split.bandpass','uid___A002_Xc660ef_X2d26.ms.split.DGCphaselow_inf'])

  if applyonly != True: es.checkCalTable('uid___A002_Xc660ef_X2d26.ms.split.DGCoffset_single', msName='uid___A002_Xc660ef_X2d26.ms.split', interactive=False)


  # apply the LF to the HF and the offset to find the phase drift between HF and LF
  # crossing scans and set solint >one DGC block - 2 solutions
  # this is an extra linear drift term
  os.system('rm -rf '+'uid___A002_Xc660ef_X2d26.ms.split.DGCoffset_single2') 
  gaincal(vis = 'uid___A002_Xc660ef_X2d26.ms.split',
    caltable = 'uid___A002_Xc660ef_X2d26.ms.split.DGCoffset_single2',
    field = '0',  #
    scan='7,9,11,13,15,39,41,43,45,47',
    spw='0,1,2,3,4,5,6,7',
    solint = '8min',  #
    refant = 'DV07',
    gaintype = 'G',
    calmode = 'p',
    minsnr=3.0,
    combine='scan',# inf and scan combine with solint setting mean 2 solutions
    interp=['linear','linearPD','linear'], 
    spwmap=[[],[8,8,8,8,8,8,8,8],[]], 
    gainfield = ['0','0','0'],  
    gaintable = ['uid___A002_Xc660ef_X2d26.ms.split.bandpass','uid___A002_Xc660ef_X2d26.ms.split.DGCphaselow_inf','uid___A002_Xc660ef_X2d26.ms.split.DGCoffset_single'])

  if applyonly != True: es.checkCalTable('uid___A002_Xc660ef_X2d26.ms.split.DGCoffset_single2', msName='uid___A002_Xc660ef_X2d26.ms.split', interactive=False)

  # apply the LF to the HF and the offset to find the phase drift between HF and LF
  # crossing scans and set solint >one DGC block - 2 solutions
  # this also uses some SPW averging as a work around to get a solution for DA54 which has some troublesome SPWs
  os.system('rm -rf '+'uid___A002_Xc660ef_X2d26.ms.split.DGCoffset_single2_DA54') 
  gaincal(vis = 'uid___A002_Xc660ef_X2d26.ms.split',
    caltable = 'uid___A002_Xc660ef_X2d26.ms.split.DGCoffset_single2_DA54',
    field = '0',  #
    scan='7,9,11,13,15,39,41,43,45,47',
    spw='0,1,2,3,4,5,6,7',
    solint = '8min',  #
    refant = 'DV07',
    gaintype = 'G',
    calmode = 'p',
    minsnr=3.0,
    combine='scan,spw',# inf and scan combine with time solint means 2 solutions
    interp=['linear','linearPD','linear'], 
    spwmap=[[],[8,8,8,8,8,8,8,8],[]], 
    gainfield = ['0','0','0'],  
    gaintable = ['uid___A002_Xc660ef_X2d26.ms.split.bandpass','uid___A002_Xc660ef_X2d26.ms.split.DGCphaselow_inf','uid___A002_Xc660ef_X2d26.ms.split.DGCoffset_single'])

  if applyonly != True: es.checkCalTable('uid___A002_Xc660ef_X2d26.ms.split.DGCoffset_single2_DA54', msName='uid___A002_Xc660ef_X2d26.ms.split', interactive=False)


#####################################
#
# Gain calibration phase cal step and amplitude calibration 
#
#####################################

mystep = 16
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  
  # Only need the INF solution for the LF as they are always cross applied, never applied to self
  os.system('rm -rf uid___A002_Xc660ef_X2d26.ms.split.phaselow_inf') 
  gaincal(vis = 'uid___A002_Xc660ef_X2d26.ms.split',
		  caltable = 'uid___A002_Xc660ef_X2d26.ms.split.phaselow_inf',
		  field = '0,2',    
		  spw='8,9,10,11',
		  solint = 'inf',  
		  refant = 'DV07',
		  combine='spw',
		  gaintype = 'G',
		  calmode = 'p',
		  minsnr=3.0,
                  gainfield=['0','0'],
		  gaintable = ['uid___A002_Xc660ef_X2d26.ms.split.bandpass','uid___A002_Xc660ef_X2d26.ms.split.DGCphaselow_phasediff'])

  if applyonly != True: es.checkCalTable('uid___A002_Xc660ef_X2d26.ms.split.phaselow_inf', msName='uid___A002_Xc660ef_X2d26.ms.split', interactive=False) 


  # Now the HF using the BP scan for phase up to set the flux scale

  os.system('rm -rf uid___A002_Xc660ef_X2d26.ms.split.phasehigh_int') 
  gaincal(vis = 'uid___A002_Xc660ef_X2d26.ms.split',
          caltable = 'uid___A002_Xc660ef_X2d26.ms.split.phasehigh_int', 
          field = '0',    
          spw='0,1,2,3,4,5,6,7',
          solint = 'int',  
          refant = 'DV07',
          gaintype = 'G',
          calmode = 'p',
          minsnr=3.0,
          gainfield='0',
          gaintable = 'uid___A002_Xc660ef_X2d26.ms.split.bandpass')
   
  if applyonly != True: es.checkCalTable('uid___A002_Xc660ef_X2d26.ms.split.phasehigh_int', msName='uid___A002_Xc660ef_X2d26.ms.split', interactive=False) 

  # Now do the amplitude cal

  os.system('rm -rf '+'uid___A002_Xc660ef_X2d26.ms.split.ampli_inf') 

  gaincal(vis = 'uid___A002_Xc660ef_X2d26.ms.split',
	  caltable = 'uid___A002_Xc660ef_X2d26.ms.split.ampli_inf',  
	  field = '0',
	  spw='0,1,2,3,4,5,6,7',
	  solint = '5min', # this will make a solution for the BP and DGC blocks independently 
	  combine='scan',  
	  refant = 'DV07',
	  gaintype = 'T',
	  calmode = 'a',
	  minsnr=3.0,
	  gaintable = ['uid___A002_Xc660ef_X2d26.ms.split.bandpass','uid___A002_Xc660ef_X2d26.ms.split.phasehigh_int'],
	  gainfield = ['0','0'])

  if applyonly != True: es.checkCalTable('uid___A002_Xc660ef_X2d26.ms.split.ampli_inf', msName='uid___A002_Xc660ef_X2d26.ms.split', interactive=False) 

  # extra solution for DA54 which uses a longer solint - i.e. to get one solutions as some of the DGC scans were flagged
  # thus is will get only a single value
  os.system('rm -rf uid___A002_Xc660ef_X2d26.ms.split.ampli_inf_DA54') 
  gaincal(vis = 'uid___A002_Xc660ef_X2d26.ms.split',
	  caltable = 'uid___A002_Xc660ef_X2d26.ms.split.ampli_inf_DA54',  
	  field = '0',
	  spw='0,1,2,3,4,5,6,7',
	  solint = '60min',
	  combine='scan',  
	  refant = 'DV07',
	  gaintype = 'T',
	  calmode = 'a',
	  minsnr=3.0,
	  gaintable = ['uid___A002_Xc660ef_X2d26.ms.split.bandpass','uid___A002_Xc660ef_X2d26.ms.split.phasehigh_int'],
	  gainfield = ['0','0'])

  if applyonly != True: es.checkCalTable('uid___A002_Xc660ef_X2d26.ms.split.ampli_inf_DA54', msName='uid___A002_Xc660ef_X2d26.ms.split', interactive=False) 


#####################################
#
# Save flags before applycal
#
#####################################

mystep = 17
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]
  
  flagmanager(vis = 'uid___A002_Xc660ef_X2d26.ms.split', mode = 'save',
              versionname = 'BeforeApplycal')


#####################################
#
# Application of gain tables 
#
#####################################

mystep = 18
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  # Extra notes:
  # We never calibrate the HF data with 'itself' for the HF data, we always use the 
  # cross apply using B2B from the LF.
  # In these data it is a 'unique' case that the BP and DGC are the same
  # this is not a necessity. However it allows us also an additional test
  # to see how well LF to HF transfer is, for application to the source itself
  # as we know its flux, and it is a strong point source 


  # Bandpass/DGC source
  applycal(vis = 'uid___A002_Xc660ef_X2d26.ms.split',
    field = '0', 
    spw='0,1,2,3,4,5,6,7', 
    interp=['linear','linearPD', 'nearest','linear','linear'],
    gaintable = ['uid___A002_Xc660ef_X2d26.ms.split.bandpass','uid___A002_Xc660ef_X2d26.ms.split.phaselow_inf','uid___A002_Xc660ef_X2d26.ms.split.DGCoffset_single','uid___A002_Xc660ef_X2d26.ms.split.DGCoffset_single2','uid___A002_Xc660ef_X2d26.ms.split.ampli_inf'], 
    spwmap = [[],[8,8,8,8,8,8,8,8],[],[],[]],
    gainfield = ['0','0','0','0','0'],
    flagbackup=False)

  #### NOTE - there is no extra correction for DA54 here - but BP is not used further ####
    
  applycal(vis = 'uid___A002_Xc660ef_X2d26.ms.split',
    field = '3,4', 
    spw='0,1,2,3,4,5,6,7', 
    interp=['linear','linearPD', 'nearest','linear','linear'],
    gaintable = ['uid___A002_Xc660ef_X2d26.ms.split.bandpass','uid___A002_Xc660ef_X2d26.ms.split.phaselow_inf','uid___A002_Xc660ef_X2d26.ms.split.DGCoffset_single','uid___A002_Xc660ef_X2d26.ms.split.DGCoffset_single2','uid___A002_Xc660ef_X2d26.ms.split.ampli_inf'], 
    spwmap = [[],[8,8,8,8,8,8,8,8],[],[],[]],
    gainfield = ['0','2','0','0','0'], # only the phase cal is mapped rest comes from DGC source
    flagbackup=False)

  # extra apply only to change the solutions for DA54 in the problematic SPWs
  applycal(vis = 'uid___A002_Xc660ef_X2d26.ms.split',
    field = '3,4', 
    spw='6,7',
    antenna='DA54&*',
    interp=['linear','linearPD', 'nearest','nearest','nearest'],
    gaintable = ['uid___A002_Xc660ef_X2d26.ms.split.bandpass','uid___A002_Xc660ef_X2d26.ms.split.phaselow_inf','uid___A002_Xc660ef_X2d26.ms.split.DGCoffset_single','uid___A002_Xc660ef_X2d26.ms.split.DGCoffset_single2_DA54','uid___A002_Xc660ef_X2d26.ms.split.ampli_inf_DA54'], 
    spwmap = [[],[8,8,8,8,8,8,8,8],[],[0,0,0,0,0,0,0,0],[]],
    gainfield = ['0','2','0','0','0'],
    flagbackup=False)


#####################################
#
# Split out corrected column
# Two versions: averaged to TDM resolution and full resolution
#
#####################################

mystep = 19
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]


  # only the TARGET VY CMa is output
  
  spws=['0','1','2','3','4','5','6','7']
  cwds=[1,1,16,16,16,16,16,16]

  # note some channel averging
  conts=[]
  for s in range(len(spws)):
      conts.append('uid___A002_Xc660ef_X2d26'[19:]+'_spw'+spws[s]+'.ms')
      os.system('rm -rf '+'uid___A002_Xc660ef_X2d26'[19:]+'_spw'+spws[s]+'.ms')
      mstransform('uid___A002_Xc660ef_X2d26.ms.split',
                  outputvis='uid___A002_Xc660ef_X2d26'[19:]+'_spw'+spws[s]+'.ms',
                  field='4',
                  datacolumn='corrected',
                  spw=spws[s],
                  regridms=True,
                  width=cwds[s],
                  outframe='lsrk')

  os.system('rm -rf vycma.cont.ms')
  concat(vis=conts,
         concatvis='vycma.cont.ms')


  # note this is not continuum subtracted however - this happens in imaging
  line=[]
  for s in range(2,len(spws)):
      line.append('uid___A002_Xc660ef_X2d26'[19:]+'ln_spw'+spws[s]+'.ms')
      os.system('rm -rf '+'uid___A002_Xc660ef_X2d26'[19:]+'ln_spw'+spws[s]+'.ms')
      mstransform('uid___A002_Xc660ef_X2d26.ms.split',
                  outputvis='uid___A002_Xc660ef_X2d26'[19:]+'ln_spw'+spws[s]+'.ms',
                  field='4',
                  datacolumn='corrected',
                  spw=spws[s],
                  regridms=True,
                  outframe='lsrk')

  os.system('rm -rf vycma.line.ms')
  concat(vis=line, concatvis='vycma.line.ms')



## END OF FILE ##
